@@ -1,14 +1,38 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """Interfaces for handling spaces."""
 from nipype.interfaces.base import (
-    traits, TraitedSpec, BaseInterfaceInputSpec, SimpleInterface,
+    traits,
+    TraitedSpec,
+    BaseInterfaceInputSpec,
+    SimpleInterface,
 )
 
 
 class _SpaceDataSourceInputSpec(BaseInterfaceInputSpec):
-    in_tuple = traits.Tuple((traits.Str, traits.Dict),
-                            mandatory=True, desc="a space declaration")
+    in_tuple = traits.Tuple(
+        (traits.Str, traits.Dict), mandatory=True, desc="a space declaration"
+    )
 
 
 class _SpaceDataSourceOutputSpec(TraitedSpec):
@@ -52,6 +76,7 @@ class SpaceDataSource(SimpleInterface):
 
     def _run_interface(self, runtime):
         from ..utils.spaces import format_reference, reference2dict
+
         self._results = reference2dict(self.inputs.in_tuple)
         self._results["uid"] = format_reference(self.inputs.in_tuple)
         return runtime
